#!/usr/bin/env python
import codecs, getopt, os, sys, shutil 
import Image
from lxml import etree

basedir = os.path.abspath(os.path.dirname(sys.argv[0]))
path = basedir+'/lib'
sys.path.insert(0, path)

import epub

urlGutenberg = 'http://gutenberg.spiegel.de'

EPUB_CSS = \
"""
@page {
  margin: 1ex 0em 1ex 0em;
}
body {
  font-family: Times, serif;
  margin: 0ex 1em 0ex 1em;
}
p {
  text-align: justify;
  margin: 0.2em 0em 0.3em 0em;
  padding: 0mm;
}

p + p {
  margin: 0.2em 0em 0.3em 0em;
  padding: 0mm;
  text-indent: 1em;
}

div.img {
  text-align: center;
  font-size: 80%;
  font-style: italic;
}

@font-face {
  font-family : Times;
  font-weight : normal;
  font-style: normal;
  src : url(times.otf);
}
@font-face {
  font-family : Times;
  font-weight : normal;
  font-style: italic;
  src : url(times-i.otf);
}
@font-face {
  font-family : Times;
  font-weight : bold;
  font-style: normal;
  src : url(times-b.otf);
}
@font-face {
  font-family : Times;
  font-weight : bold;
  font-style: italic;
  src : url(times-bi.otf);
}

"""

def copyTree(tgt, src):
  """
  Copy element tree without links.
  """
  tgt.text = src.text
  for n in src:
    if n.tag == 'a':
      text = ''
      if 'class' in n.attrib and n.attrib['class'] == 'zenoTXLinkInt':
        text = '[%s]' % n.text
	span = etree.SubElement(tgt, 'span', attrib={'class': 'footnote'})
	copyTree(span, n)
	continue
      if n.tail is None:
        continue
      if len(tgt) == 0:
        if tgt.text is None:
	  tgt.text = n.tail
	else:
	  tgt.text += n.tail
      else:
        if tgt[-1].tail is None:
	  tgt[-1].tail = n.tail
	else:
	  tgt[-1].tail += n.tail
      continue
    # Ignore comment and other special nodes
    if type(n).__name__ == '_Element':
      nt = etree.SubElement(tgt, n.tag)
      for a in n.attrib:
        nt.attrib[a] = n.attrib[a]
      copyTree(nt, n)
  tgt.tail = src.tail

class GutenbergParser(epub.EpubParser):

  def getImgFile(self, url):
    self.imgCounter += 1
    filename = "image-%d" % self.imgCounter
    path = "%s/%s" % (self.dataDir, filename)
    epub.wget("%s%s" % (self.base, url), path)
    img = Image.open(path)
    if img.format in epub.imgTypes:
      newFileName = filename+epub.imgTypes[img.format][0]
      os.rename(path, "%s/%s" % (self.dataDir, newFileName))
      self.resources["image-%d" % self.imgCounter] = \
        [newFileName, epub.imgTypes[img.format][1]]
      return img, newFileName
    else:
      return None, None

  def handleImg(self, node, res):
    href = node.attrib['src']
    print "Get Image:",href
    img, name = self.getImgFile(href)
    if img is None:
      return
    div = etree.SubElement(res, 'div', attrib={'class': 'img'})
    nimg = etree.SubElement(div, 'img')
    nimg.attrib['src'] = name
    nimg.attrib['alt'] = name
    nimg.attrib['width'] = str(img.size[0])
    nimg.attrib['height'] = str(img.size[1])
    return div

  def getTitleL(self, nodelist):
    res = []
    for n in nodelist:
      res.append(n.text)
    return " ".join(res)

  def parseTitle(self, node, cnt):
    div = node.xpath('.//div[@id="metadata"]')[0]
    #ntitle = div.xpath('./gb_meta[@name="title"]')[0]
    #nauthor = div.xpath('./gb_meta[@name="author"]')[0]
    #title = ntitle.attrib['content']
    #author = nauthor.attrib['content']
    meta = {'author': 'Autor', 'title': 'Titel'}
    for row in div.xpath('table/tr'):
      col0, col1 = row.xpath('td')
      for x in ('author', 'title'):
        if col0[0].text == x:
          print x, col0[0].text, col1.text
          meta[x] = col1.text
    html = self.createHtml(meta['title'])
    body = etree.SubElement(html, 'body')
    nh3 = etree.SubElement(body, 'h3', attrib={'class': 'author'})
    nh3.text = meta['author']
    nh2 = etree.SubElement(body, 'h2', attrib={'class': 'title'})
    nh2.text = meta['title']

    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    self.sections.append([id, '%s.xhtml' % id, 'Titelseite'])
    self.title = meta['title']
    self.author = meta['author']
  
  def parsePart(self, node, cnt):
    h3 = node.xpath('.//h3')
    title = self.getTitleL(h3)
    html = self.createHtml(title)
    body = etree.SubElement(html, 'body')
    nh4 = etree.SubElement(body, 'h3', attrib={'class': 'part'})
    nh4.text = title
    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    self.sections.append([id, '%s.xhtml' % id, title])

  def parseChapter(self, nmeta, node, cnt):
    #print nmeta
    #ntitle = nmeta.xpath("gb_meta[@name='title']")[0]
    #title = ntitle.attrib['content']
    title = 'Chapter Title'
    html = self.createHtml(title)
    body = etree.SubElement(html, 'body')
    state = 'title'
    ntitle = None
    title = u''
    for n in node:
      if n.tag == 'a' and state == 'title': # ignore links in title part
        continue
      if n.tag == 'h3':
        if state == 'title':
	  if ntitle == None:
	    ntitle = etree.SubElement(body, 'h4', attrib={'class': 'chapter'})
	    ntitle.text = n.text
	    title = ntitle.text
	  else:
	    br = etree.SubElement(ntitle, 'br')
	    br.tail = n.text
	    title += ' '+n.text
	else:
	  nh3 = etree.SubElement(body, 'h3')
	  nh3.text = n.text
      elif n.tag == 'h4':
        if state == 'title':
	  if ntitle == None:
	    ntitle = etree.SubElement(body, 'h4', attrib={'class': 'chapter'})
	    ntitle.text = n.text
	    title = ntitle.text
	  else:
	    br = etree.SubElement(ntitle, 'br')
	    br.tail = n.text
	    title += ' '+n.text
	else:
	  nh4 = etree.SubElement(body, 'h4')
	  nh4.text = n.text
      elif n.tag == 'p':
        p = etree.SubElement(body, 'p')
	copyTree(p, n)
	state = 'body'
      elif n.tag == 'div' and n.attrib['class'] == 'figure':
        nImg = n.xpath('.//img')
        if len(nImg) > 0:
          img = nImg[0]
          self.handleImg(img, body)
        else:
          p = etree.SubElement(body, 'p')
          copyTree(p, n)
	state = 'body'
      elif n.tag == 'img':
	self.handleImg(n, body)
	state = 'body'
      else:
        state = 'body'

    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    if title == '':
      title = 'Kapitel %d' % cnt
    self.sections.append([id, '%s.xhtml' % id, title])

  def parseFile(self, filename, cnt):
    print "Parse %s" % filename
    parser = etree.HTMLParser()
    doc = etree.parse(filename,parser)
    root = doc.getroot()
    #meta = root.xpath('.//gb_meta')
    #for m in meta:
    #  print etree.tostring(m, pretty_print=True)
    meta = root.xpath('.//div[@id="metadata"]')[0]
    div = root.xpath('.//div[@id="gutenb"]')[0]
    nbase = root.xpath('/html/head/base')[0]
    self.base = nbase.attrib['href']
    #print etree.tostring(div, pretty_print=True)
    h2 = div.xpath('.//h2')
    h3 = div.xpath('.//h3')
    h4 = div.xpath('.//h4')
    if cnt == 1:
      self.parseTitle(root, cnt-1)
    type = 'chapter'
    self.parseChapter(meta, div, cnt)
#    elif len(h4) > 0:
#      type = 'chapter'
#      self.parseChapter(div, cnt)
#      h4 = h4[0]
#      print "Chapter %d %s" % (cnt, h4.text)
#      for node in div.xpath('p|div[@class="ZenoIMBreak"]'):
#        #print etree.tostring(node)
#	pass

  def parseFileList(self, filename):
    for f in ['epub', 'epub/META-INF', 'epub/data']:
      if not os.path.exists(f):
	os.mkdir(f)
    self.dataDir = 'epub/data'
    cnt = 0
    for line in open(filename):
      cnt += 1
      f = line.strip()
      self.parseFile(f, cnt)
    self.writeCss()
    f = open('epub/mimetype', 'w')
    f.write('application/epub+zip\n')
    f.close()
    self.copyFonts()
    self.writeContainer()
    uid = self.writePackage()
    self.writeToc(uid)
    epub.doZip(targetfile)

def usage():
  print "Usage: %s [-f] targetfile filelist" % sys.argv[0]
  print " targetfile without extension"
  print " -f embed Times font with unicode symbols"

if __name__ == '__main__':
  if len(sys.argv) < 3:
    usage()
    sys.exit(1)
  optlist, args = getopt.getopt(sys.argv[1:], 'f')
  targetfile = args[0]
  filename = args[1]
  embedFont = False
  for k, v in optlist:
    if k == '-f':
      embedFont = True

  GutenbergParser(basedir, targetfile, embedFont).parseFileList(filename)
