#!/usr/bin/env python

import codecs, getopt, os, sys, shutil
import Image
from lxml import etree

basedir = os.path.abspath(os.path.dirname(sys.argv[0]))
path = basedir+'/lib'
sys.path.insert(0, path)
HOST = "http://www.zeno.org"

import epub

def copyTree(tgt, src):
  """
  Copy element tree without links.
  """
  tgt.text = src.text
  for n in src:
    if n.tag == 'a':
      text = ''
      if 'class' in n.attrib and n.attrib['class'] == 'zenoTXLinkInt':
        text = '[%s]' % n.text
	span = etree.SubElement(tgt, 'span', attrib={'class': 'footnote'})
	copyTree(span, n)
	continue
      if n.tail is None:
        continue
      if len(tgt) == 0:
        if tgt.text is None:
	  tgt.text = n.tail
	else:
	  tgt.text += n.tail
      else:
        if tgt[-1].tail is None:
	  tgt[-1].tail = n.tail
	else:
	  tgt[-1].tail += n.tail
      continue
    nt = etree.SubElement(tgt, n.tag)
    for a in n.attrib:
      nt.attrib[a] = n.attrib[a]
    copyTree(nt, n)
  tgt.tail = src.tail

class ZenoParser(epub.EpubParser):

  def getImgFile(self, url):
    self.imgCounter += 1
    filename = "image-%d" % self.imgCounter
    path = "%s/%s" % (self.dataDir, filename)
    epub.wget(url, path)
    img = Image.open(path)
    if img.format in epub.imgTypes:
      newFileName = filename+epub.imgTypes[img.format][0]
      os.rename(path, "%s/%s" % (self.dataDir, newFileName))
      self.resources["image-%d" % self.imgCounter] = \
        [newFileName, epub.imgTypes[img.format][1]]
      return img, newFileName
    else:
      return None, None

  def handleImg(self, node, res):
    href = node.attrib['src']
    print href
    if href[0] == '/':
      href = HOST+href
    img, name = self.getImgFile(href)
    if img is None:
      return
    div = etree.SubElement(res, 'div', attrib={'class': 'img'})
    nimg = etree.SubElement(div, 'img')
    nimg.attrib['src'] = name
    nimg.attrib['alt'] = name
    nimg.attrib['width'] = str(img.size[0])
    nimg.attrib['height'] = str(img.size[1])
    return div

  def getTitleL(self, nodelist):
    res = []
    for n in nodelist:
      res.append(n.text)
    return " ".join(res)

  def parseTitle(self, node, cnt):
    h2 = node.xpath('.//h2')
    h3 = node.xpath('.//h3')
    h4 = node.xpath('.//h4')
    title = self.getTitleL(h2)
    author = self.getTitleL(h3)
    html = self.createHtml(title)
    body = etree.SubElement(html, 'body')
    if len(h3) > 0:
      nh3 = etree.SubElement(body, 'h3', attrib={'class': 'author'})
      nh3.text = author
    nh2 = etree.SubElement(body, 'h2', attrib={'class': 'title'})
    nh2.text = title
    for ph4 in h4:
      nh4 = etree.SubElement(body, 'h4', attrib={'class': 'subtitle'})
      nh4.text = ph4.text
    # Bilder
    #for n in node.xpath('div[@class="ZenoIMBreak"]'):
    for n in node.xpath('div[@class="zenoIMBreak"]'):
      img = n.xpath('.//img')[0]
      self.handleImg(img, body)

    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    self.sections.append([id, '%s.xhtml' % id, 'Titelseite'])
    self.title = title
    self.author = author
  
  def parsePart(self, node, cnt):
    h3 = node.xpath('.//h3')
    title = self.getTitleL(h3)
    html = self.createHtml(title)
    body = etree.SubElement(html, 'body')
    nh4 = etree.SubElement(body, 'h3', attrib={'class': 'part'})
    nh4.text = title
    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    self.sections.append([id, '%s.xhtml' % id, title])

  def parseChapter(self, node, cnt):
    h4 = node.xpath('.//h4')[0]
    title = h4.text
    html = self.createHtml(title)
    body = etree.SubElement(html, 'body')
    ntitle = etree.SubElement(body, 'h4', attrib={'class': 'chapter'})
    ntitle.text = title
    n = h4.getnext()
    state = 'title'
    while not n is None:
      if n.tag == 'h4':
        if state == 'title':
	  br = etree.SubElement(ntitle, 'br')
	  br.tail = n.text
	  title += ' ' + n.text
	else:
	  nh4 = etree.SubElement(body, 'h4')
	  nh4.text = n.text
      elif n.tag == 'div' and n.attrib['class'] == 'zenoIMBreak':
	img = n.xpath('.//img')[0]
	self.handleImg(img, body)
	state = 'body'
      elif n.tag == 'div' and n.attrib['class'] == 'zenoCOGallery':
        for sdiv in n.xpath('.//div[@class="zenoTH"]'):
          img = sdiv.xpath('.//img')[0]
	  txt = sdiv.xpath('.//div[@class="zenoTHText"]')[0]
	  ndiv = self.handleImg(img, body)
	  br = etree.SubElement(ndiv, 'br')
	  br.tail = txt.text
	state = 'body'
      elif n.tag == 'p':
        p = etree.SubElement(body, 'p')
	copyTree(p, n)
	state = 'body'
      else:
        state = 'body'
      n = n.getnext()

    id = 'part-%d' % cnt
    f = open('%s/%s.xhtml' % (self.dataDir, id), 'w')
    self.writeDocType(f)
    self.doc.write(f, encoding='UTF-8',pretty_print=True)
    f.close()
    self.resources[id] = ['%s.xhtml' % id, 'application/xhtml+xml']
    self.sections.append([id, '%s.xhtml' % id, title])

  def parseFile(self, filename, cnt):
    print "Parse %s" % filename
    parser = etree.HTMLParser()
    doc = etree.parse(filename,parser)
    root = doc.getroot()
    div = root.xpath('.//div[@class="zenoCOMain"]')[0]
    h2 = div.xpath('.//h2')
    h3 = div.xpath('.//h3')
    h4 = div.xpath('.//h4')
    if len(h2) > 0:
      type = 'title'
      self.parseTitle(div, cnt)
    elif len(h3) > 0:
      type = 'part'
      self.parsePart(div, cnt)
    elif len(h4) > 0:
      type = 'chapter'
      self.parseChapter(div, cnt)
      h4 = h4[0]
      print "Chapter %d %s" % (cnt, h4.text)
      for node in div.xpath('p|div[@class="ZenoIMBreak"]'):
        #print etree.tostring(node)
	pass
    print type

  def parseFileList(self, filename):
    for f in ['epub', 'epub/META-INF', 'epub/data']:
      if not os.path.exists(f):
	os.mkdir(f)
    self.dataDir = 'epub/data'
    cnt = 0
    for line in open(filename):
      cnt += 1
      f = line.strip()
      self.parseFile(f, cnt)
    self.writeCss()
    f = open('epub/mimetype', 'w')
    f.write('application/epub+zip\n')
    f.close()
    self.copyFonts()
    self.writeContainer()
    uid = self.writePackage()
    self.writeToc(uid)
    epub.doZip(targetfile)

def usage():
  print "Usage: %s targetfile filelist" % sys.argv[0]
  print " targetfile without extension"
  print " -f embed Times font with unicode symbols"

if __name__ == '__main__':
  if len(sys.argv) < 3:
    usage()
    sys.exit(1)
  optlist, args = getopt.getopt(sys.argv[1:], 'f')
  targetfile = args[0]
  filename = args[1]
  embedFont = False
  for k, v in optlist:
    if k == '-f':
      embedFont = True
  ZenoParser(basedir, targetfile, embedFont).parseFileList(filename)
